@@ -1,14 +1,14 @@
 #!/usr/bin/env python3
 
-import cmd, ast
+import cmd
 import argparse
-import json
 from prompt_toolkit import PromptSession
+from prompt_toolkit.key_binding import KeyBindings
 from prompt_toolkit.lexers import PygmentsLexer
 from pygments.lexers import SqlLexer
 from influxdb_client_3 import InfluxDBClient3
-import os
-from config_helper import config_helper
+from helper import config_helper
+from openai_helper import OpenAIHelper
 
 _usage_string = """
 to write data use influxdb line protocol:
@@ -24,11 +24,11 @@ to enter interactive mode:
 _description_string = 'CLI application for Querying IOx with arguments and interactive mode.'
 
 class IOXCLI(cmd.Cmd):
-    intro = 'Welcome to my IOx CLI.\n'
+    intro = 'InfluxDB 3.0 CLI.\n'
     prompt = '(>) '
 
     def __init__(self):
-        super().__init__()
+        super(IOXCLI, self).__init__()
         self.config_helper = config_helper()
         self.active_config = self.config_helper._get_active()
         self._setup_client()
@@ -36,25 +36,66 @@ class IOXCLI(cmd.Cmd):
         self._write_prompt_session = PromptSession(lexer=None)
        
 
-    def do_sql(self, arg):
-        if self.active_config == {}:
-            print("can't query, no active configs")
-            return
-        try: 
-            table = self.influxdb_client.query(query=arg, language="sql")
-            print(table.to_pandas().to_markdown())
-        except Exception as e:
-            print(e)
+    
 
-    def do_influxql(self, arg):
+    def do_query(self, arg, language):
         if self.active_config == {}:
             print("can't query, no active configs")
             return
-        try: 
-            table = self.influxdb_client.query(query=arg, language="influxql")
-            print(table.to_pandas().to_markdown())
+        
+        # Retrieve the iterator
+        reader = self._query_chunks(arg, language)
+
+        if reader is None:
+            return
+
+        # Create custom key bindings
+        bindings = KeyBindings()
+
+        # Flag to determine if there is more data
+        has_more_data = True
+
+        # Bind the 'tab' key
+        @bindings.add('tab')
+        def _(event):
+            nonlocal has_more_data
+            if has_more_data:
+                try:
+                    batch, buff = reader.read_chunk()
+                    print(batch.to_pandas().to_markdown())
+                except StopIteration:
+                    print("End of data. Press Enter to continue.")
+                    has_more_data = False
+                except Exception as e:
+                    print(e)
+                    has_more_data = False
+
+        # Create a session with the bindings
+        session = PromptSession(key_bindings=bindings)
+        try:
+            batch, buff = reader.read_chunk()
+            print(batch.to_pandas().to_markdown())
+        except StopIteration:
+            print("End of data. Press Enter to continue.")
+            has_more_data = False
+  
+        while True:
+            try:
+                if not has_more_data:
+                    break
+                # Prompt loop to capture key presses
+                print("Press TAB to fetch next chunk of data")
+                session.prompt()
+            except KeyboardInterrupt:
+                break
+    
+    def _query_chunks(self, arg, language):
+        try:
+            table = self.influxdb_client.query(query=arg, language=language, mode="chunk")
+            return table
         except Exception as e:
             print(e)
+            return None
 
     def do_write(self, arg):
         if self.active_config == {}:
@@ -66,7 +107,7 @@ class IOXCLI(cmd.Cmd):
         
         self.influxdb_client.write(record=arg)
     
-    def do_write_csv(self, args):
+    def do_write_file(self, args):
         if self.active_config == {}:
             print("can't write, no active configs")
             return
@@ -81,12 +122,27 @@ class IOXCLI(cmd.Cmd):
                 temp[attribute] = arg_value
         if isinstance(temp['tags'], str):
             temp['tags'] =  temp['tags'].split(',')
+        
 
-
-        self.influxdb_client.write_csv(csv_file=temp['file'], 
+        if 'measurement' in temp:
+            try:
+                self.influxdb_client.write_file(file=temp['file'], 
                                        measurement_name=temp['measurement'], 
                                        timestamp_column=temp['time'], 
                                        tag_columns=temp['tags'])
+                
+            except Exception as e:
+                print(e)
+        else:
+            print("measurement not specified. Attempting to find measurement in file...")
+            try:
+                self.influxdb_client.write_file(file=temp['file'], 
+                                       timestamp_column=temp['time'], 
+                                       tag_columns=temp['tags'])
+                
+            except Exception as e:
+                print(e)
+                
 
     def do_exit(self, arg):
         'Exit the shell: exit'
@@ -97,17 +153,30 @@ class IOXCLI(cmd.Cmd):
         'Exit the shell with Ctrl-D'
         return self.do_exit(arg)
 
+    def do_chatgpt(self, arg):
+        if arg == "":
+            print("can't write, no line protocol supplied")
+            return
+        openai_helper = OpenAIHelper()
+        query = openai_helper.nl_to_sql(arg)
+        print(f"Run InfluxQL query: {query}")
+        self.do_query(query, language='influxql')
+        
+
  
     def precmd(self, line):
         if line.strip() == 'sql':
-            self._run_prompt_loop('(sql >) ', self.do_sql, 'SQL mode')
+            self._run_prompt_loop('(sql >) ', lambda arg: self.do_query(arg, language='sql'), 'SQL mode')
             return ''
         if line.strip() == 'influxql':
-            self._run_prompt_loop('(influxql >) ', self.do_influxql, 'INFLUXQL mode')
+            self._run_prompt_loop('(influxql >) ', lambda arg: self.do_query(arg, language='influxql'), 'INFLUXQL mode')
             return ''
         if line.strip() == 'write':
             self._run_prompt_loop('(write >) ', self.do_write, 'write mode')
             return ''
+        if line.strip() == 'chatgpt':
+            self._run_prompt_loop('(chatgpt >) ', self.do_chatgpt, 'chatgpt mode')
+            return ''
         return line
 
     def _run_prompt_loop(self, prompt, action, mode_name):
@@ -173,14 +242,17 @@ def parse_args():
     influxql_parser = subparsers.add_parser('influxql', help='execute the given InfluxQL query')
     influxql_parser.add_argument('query', metavar='QUERY', nargs='*', action=StoreRemainingInput, help='the INFLUXQL query to execute')
 
+    chatgpt_parser = subparsers.add_parser('chatgpt', help='execute the given chatgpt statement')
+    chatgpt_parser.add_argument('query', metavar='QUERY', nargs='*', action=StoreRemainingInput, help='the chatgpt query to execute')
+
     write_parser = subparsers.add_parser('write', help='write line protocol to InfluxDB')
     write_parser.add_argument('line_protocol', metavar='LINE PROTOCOL',  nargs='*', action=StoreRemainingInput, help='the data to write')
 
-    write_csv_parser = subparsers.add_parser('write_csv', help='write CSV data to InfluxDB')
-    write_csv_parser.add_argument('--file', help='the CSV file to import', required=True)
-    write_csv_parser.add_argument('--measurement', help='Define the name of the measurement', required=True)
-    write_csv_parser.add_argument('--time', help='Define the name of the time column with the csv file', required=True)
-    write_csv_parser.add_argument('--tags', help='(optional) array of column names which are tags. Format should be: ["tag1", "tag2"]', required=False)
+    write_file_parser = subparsers.add_parser('write_file', help='write data from file to InfluxDB')
+    write_file_parser.add_argument('--file', help='the file to import', required=True)
+    write_file_parser.add_argument('--measurement', help='Define the name of the measurement', required=False)
+    write_file_parser.add_argument('--time', help='Define the name of the time column within the file', required=True)
+    write_file_parser.add_argument('--tags', help='(optional) array of column names which are tags. Format should be: ["tag1", "tag2"]', required=False)
 
     config_parser = subparsers.add_parser("config", help="configure the application")
     config_subparsers = config_parser.add_subparsers(dest='config_command')
@@ -221,13 +293,15 @@ def main():
 
 
     if args.command == 'sql':
-        app.do_sql(args.query)
+        app.do_query(args.query, language='sql')
     if args.command == 'influxql':
-        app.do_influxql(args.query)
+        app.do_query(args.query, language='influxql')
+    if args.command == 'chatgpt':
+        app.do_chatgpt(args.query)
     if args.command == 'write':
         app.do_write(args.line_protocol)
-    if args.command == 'write_csv':
-        app.do_write_csv(args)
+    if args.command == 'write_file':
+        app.do_write_file(args)
     if args.command == 'config':
         if args.config_command == 'create':
             app.create_config(args)
